﻿using Http.Core;
using PCUT.Entities;
using PCUT.Models;
using System;
using System.Threading.Tasks;
using Http.Core.Extensions;
using static Http.Core.Constants.HttpConstants;
using Http.Core.Contexts;
using Windows.UI.Xaml;
using Http.Core.Exceptions;
using PCUT.Entities.ApiResponse;
using System.Collections.Generic;
using System.Text;
using Windows.Storage;
using System.Linq;

namespace PCUT.ViewModels
{
    public class MainMenuPageViewModel : NotificationBase
    {
        public const string BannerFile = "banner.html";

        private string _userName;
        public string UserName
        {
            get => _userName;
            set => SetProperty(ref _userName, value);
        }

        private string _expiry;
        public string Expiry
        {
            get => _expiry;
            set => SetProperty(ref _expiry, value);
        }

        private bool _isUserRoleAdmin = false;
        public bool IsUserRoleAdmin
        {
            get => _isUserRoleAdmin;
            set
            {
                if (_isUserRoleAdmin != value)
                {
                    _isUserRoleAdmin = value;
                    RaisePropertyChanged(nameof(IsUserRoleAdmin));
                }
            }
        }

        private bool _isMenuEnabled = false;
        public bool IsMenuEnabled
        {
            get => _isMenuEnabled;
            set
            {
                SetProperty(ref _isMenuEnabled, value);
            }
        }

        private Visibility _isMenuDisplay = Visibility.Collapsed;
        public Visibility IsMenuDisplay
        {
            get => _isMenuDisplay;
            set
            {
                if (_isMenuDisplay != value)
                {
                    SetProperty(ref _isMenuDisplay, value);
                }
            }
        }

        public SelectableCollection<Category> Categories { get; }

        public MainMenuPageViewModel()
        {
            IsUserRoleAdmin = UserContext.Instance.Profile?.Role?.Equals("admin", StringComparison.OrdinalIgnoreCase) ?? false;
            Categories = new SelectableCollection<Category>(new Category { Name = "-None-" });
        }

        public void ToggleMenuDisplay()
        {
            IsMenuDisplay = IsMenuDisplay == Visibility.Collapsed ? Visibility.Visible : Visibility.Collapsed;
        }

        public void HideMenuDisplay()
        {
            IsMenuDisplay = Visibility.Collapsed;
        }

        public async Task GetCategoriesAsync()
        {
            using (var client = HttpClientFactory.CreateClient(ClientNames.ApiClient))
            {
                var categories = await client.GetCategoriesAsync(filterByRole: true);
                Categories.Load(categories);
            }
        }

        public async Task<bool> GetBannersAsync()
        {
            using (var client = HttpClientFactory.CreateClient(ClientNames.ApiClient))
            {
                try
                {
                    var response = await client.GetAsync($"{Api.Banner}?page=1&pageSize={int.MaxValue}");
                    if (response.IsSuccessStatusCode)
                    {
                        var apiResponse = await response.DeserializeObjectAsync<PaginationResponse<IEnumerable<Banner>>>();
                        var banners = apiResponse.Data;
                        var bannersCount = banners.Count();
                        var contentBuilder = new StringBuilder();
                        contentBuilder.Append(
$@"<!DOCTYPE html><html>
<head>
<meta charset=""UTF-8"">
<style>
::-webkit-scrollbar {{background: transparent;}}
.marquee-container {{display: flex;overflow: visible;white-space: nowrap;animation: marquee {bannersCount*20}s linear infinite;}}
.marquee-text {{width: 100%;margin-right: 10%;font-family: Arial, sans-serif;}}
@keyframes marquee {{0% {{transform: translateX(100%);}} 100% {{transform: translateX(-{bannersCount*110}%);}}}}
</style></head>
<body style=""-ms-overflow-style: none;scrollbar-width: none;""><div class=""marquee-container"">");
                        foreach (var banner in banners)
                        {
                            contentBuilder.Append(@"<div class=""marquee-text"">").Append(banner.Content).Append("</div>");
                        }
                        contentBuilder.Append(@"</div></body></html>");
                        StorageFolder folder = await ApplicationData.Current.TemporaryFolder.CreateFolderAsync("banner", CreationCollisionOption.OpenIfExists);
                        StorageFile file = await folder.CreateFileAsync(BannerFile, CreationCollisionOption.ReplaceExisting);
                        await FileIO.WriteTextAsync(file, contentBuilder.ToString());
                        return true;
                    }
                }
                catch (Exception ex) when (!(ex is AppOutdatedException))
                {
                }
                return false;
            }
        }

        public string DataCenterSearchValue { get; set; }
    }
}
